-- File-Revision: 13
-- cooldown stuff

local find = string.find

RCOptions = {};

local defaults = {disableBuff = false, disableDebuff = false};

SLASH_RANGECOOLDOWN1, SLASH_RANGECOOLDOWN2 = '/rangecooldown', '/rc'
function SlashCmdList.RANGECOOLDOWN(msg, editbox)
  local command, rest = msg:match("^(%S*)%s*(.-)$");
  
  if command == "buff" then
    if rest == "show" then
      RCOptions.disableBuff = false;
      RCPrintMessage("Buff Cooldowns Enabled");
    else
      RCOptions.disableBuff = true;
      RCPrintMessage("Buff Cooldowns Disabled");
    end  
  elseif command == "debuff" then
    if rest == "show" then
      RCOptions.disableDebuff = false;
      RCPrintMessage("Debuff Cooldowns Enabled");
    else
      RCOptions.disableDebuff = true;
      RCPrintMessage("Debuff Cooldowns Disabled");
    end
  else
    RCPrintMessage("Syntax: /rangecooldown (buff|debuff) (show||hide)");
  end
end

function RCPrintMessage(msg)
  DEFAULT_CHAT_FRAME:AddMessage("|cFF00FF00RangeCooldown: |r"..msg, .2, .4, 1.0);
end

-- updates the timer text if there is some and checks for a new timer if we need to
function cooldownTimer_OnUpdate(self, elapsed)
  self.elapsed = self.elapsed + elapsed;
  if self.elapsed > .1 then
    cooldownTimer_SetTime(self);
--    cooldownTimer_CheckTimer(self);
    self.elapsed = 0;
    if self.check then
      local action = ((self:GetParent()):GetParent()).action
      cooldownTimer_CheckTimer(self, action);
    end
  end
end

-- Formats and then sets the time on the given timer
function cooldownTimer_SetTime(timer)
  local timeleft = timer.time - GetTime();

  if timeleft < 0 then
    timer.time = 0;
    timer.text:SetText("");
  elseif timeleft < 1 then
    timer.text:SetFontObject("CooldownFontSmall");
    timer.text:SetText(("%.1f"):format(timeleft));
  elseif timeleft < 60 then
    timer.text:SetFontObject("CooldownFontLarge");
    timer.text:SetText(("%d"):format(timeleft));
  elseif timeleft < 3600 then
    timeleft = floor(timeleft/60);
    timer.text:SetFontObject("CooldownFontLarge");
    timer.text:SetText(("%dm"):format(timeleft));
  else
    timeleft = floor(timeleft/3600);
    timer.text:SetFontObject("CooldownFontLarge");
    timer.text:SetText(("%dh"):format(timeleft));
  end
end

-- function to make sure the actionbuttons have timers created for them
function cooldownTimer_UpdateAction(button)
  local timer = _G[button:GetName().."CooldownTimer"]
  timer = timer or cooldownTimer_CreateTimer(button.cooldown);
   if button.action and HasAction(button.action) then
    cooldownTimer_CheckTimer(timer, action);
    timer:Show();
  else
    timer:Hide();
  end
end

-- funtion hooked to the set timer function for cooldowns.  gets called for items and most cooldowns except action buttons.
function cooldownTimer_SetTimer(self, start, duration, enable, charges, maxcharges)
  --local button = self:GetParent();
  local name = self:GetName(); 

  -- grab the timer object for the given button
  local timer = _G[name.."Timer"];

  if (RCOptions.disableBuff and find(name,'Buff')) or (RCOptions.disableDebuff and find(name,"Debuff")) then
    if timer then
      timer.text:Hide();
    end
    return;
  end

   -- make sure we have valid timer, create one if necessary
  timer = timer or cooldownTimer_CreateTimer(self);

  -- if we have a start time and the duration is over the base gcd of 1.5 then set the time
  if (start and start > 0 and duration > 1.5 and enable > 0) then
    timer.time = duration + start;
    timer:Show();
    cooldownTimer_SetTime(timer);
    timer.check = nil;
  else
    timer:Hide();
  end
end

do 
  hooksecurefunc("ActionButton_Update",cooldownTimer_UpdateAction);
  hooksecurefunc("CooldownFrame_SetTimer",cooldownTimer_SetTimer);
end

-- Create a timer frame for the given cooldown frame
function cooldownTimer_CreateTimer(self)
  local frameName = self:GetName().."Timer";
  
  local frame = CreateFrame("FRAME",frameName,self,"CooldownTimerTemplate");
  
  if (self:GetParent()).cooldown then
    frame:RegisterEvent("ACTIONBAR_UPDATE_STATE");
  end
  
  frame.text:SetText("");
  frame.text:SetTextColor(1.0,0.1,0.1);
  
  frame.time = 0;
  frame.elapsed = 0;
  
    -- if we are a debuff lets make the text smaller to fit in the smaller icon
  if find(frameName,'Buff') or find(frameName,"Debuff") then
    frame:SetScale(.5);
  end
  
  return frame;
end

-- When a cooldown frame is shown check for the cooldown that needs to be shown
function cooldownTimer_OnShow(self, ...)
  self.check = true;
end

-- function called when the actionbar cooldown infomation has been updated
function cooldownTimer_OnEvent(self, event, arg1)
  local action = ((self:GetParent()):GetParent()).action
  cooldownTimer_CheckTimer(self, action);
end

-- Checks the given timer to see if the action given has a cooldown that needs to be shown
function cooldownTimer_CheckTimer(timer, action)

  -- if there is an action sent in and its has a valid action attached
  if action and HasAction(action) then
    local start, duration, enable, charges, maxcharges = GetActionCooldown(action);
    
    -- if the cooldown is enabled and has a valid start
    if enable and enable > 0 and start and start > 0 then
      -- if the duration is over 1.5s then show the text
      if duration > 1.5 then
        timer.time = duration + start;
        timer.text:Show();
        cooldownTimer_SetTime(timer);
      else
        -- otherwise hide the text
        timer.text:Hide();
      end
      
      -- we found the cd so no need to check more
      timer.check = nil;
    else
      -- no cooldown so hide text
      timer.text:Hide();
    end
  end
end

function cooldownTimerFrame_OnEvent(self, event, arg1)
  if arg1 == "RangeCooldown" then
    -- load our defaults if no settings are found
    local key, value;
    for key, value in pairs(defaults) do
      if RCOptions[key] == nil then
        RCOptions[key] = value;
      end
    end
  end
end